import React from 'react';
import PropTypes from 'prop-types';
import { Router } from 'dva/router';
import config from './utils/config';
import { isAuthed, hasDomain } from './utils/auth';
import { fullPath, makePromise0 } from './utils/helper';
import App from './routes/app';
import { foreachModule, getChildren } from './data/modules';
import Monk from './routes/main/monk';

const { contextPath } = config;

const registerModel = (app, model) => {
// eslint-disable-next-line no-underscore-dangle
  if (!(app._models.filter(m => m.namespace === model.namespace).length === 1)) {
    app.model(model);
  }
};

const maybeLogin = (nextState, replace) => {
  if (!isAuthed()) {
    return replace(fullPath('/login'));
  }
};

const maybeSwitch = (nextState, replace) => {
  if (!hasDomain()) {
    return replace(fullPath('/domain'));
  }
};

const authenticated = (nextState, replace) => {
  maybeLogin(nextState, replace);
  maybeSwitch(nextState, replace);
};

const createRoute = async (app, module) => {
  const route = {
    path: module.id,
    name: module.name,
  };
  if (!module.group) {
    if (module.route) {
      const modelBundle = await makePromise0(require('bundle!./models/main/modules/' + module.route + '/index.js')); // eslint-disable-line import/no-dynamic-require, prefer-template, global-require
      registerModel(app, modelBundle);
      const routeBundle = await makePromise0(require('bundle!./routes/main/modules/' + module.route + '/index.js')); // eslint-disable-line import/no-dynamic-require, prefer-template, global-require
      route.component = routeBundle;
      if (routeBundle.route) {
        for (const key in routeBundle.route) {
          if ({}.hasOwnProperty.call(routeBundle.route, key)) {
            route[key] = routeBundle.route[key];
          }
        }
      }
    } else {
      route.component = Monk;
    }
  } else {
    route.component = Monk;
    route.getChildRoutes = (nextState, cb) => {
      createModuleRoutes(app, getChildren(module.id))
        .then((result) => {
          cb(null, result);
        })
        .catch((err) => {
          cb(err, null);
        });
    };
  }
  return route;
};

const createModuleRoutes = async (app, modules) => {
  const routes = [];
  for (const module of modules) {
    routes.push(await createRoute(app, module));
  }
  routes.push({
    path: '*',
    onEnter(nextState, replace) {
      replace(fullPath('/main'));
    },
  });
  return routes;
};

function RouterConfig({ history, app }) {
  const routes = [
    {
      path: `${contextPath}/`,
      component: App,
      indexRoute: {
        onEnter: (nextState, replace) => replace(`${contextPath}/main`),
      },
      childRoutes: [
        {
          path: 'login',
          getComponent(nextState, cb) {
            require.ensure([], (require) => {
              registerModel(app, require('./models/login'));
              cb(null, require('./routes/login'));
            }, 'login');
          },
        },
        {
          path: 'domain',
          onEnter: maybeLogin,
          getComponent(nextState, cb) {
            require.ensure([], (require) => {
              registerModel(app, require('./models/domain'));
              cb(null, require('./routes/domain'));
            }, 'domain');
          },
        },
        {
          path: 'main',
          onEnter: authenticated,
          getComponent(nextState, cb) {
            require.ensure([], (require) => {
              registerModel(app, require('./models/main'));
              cb(null, require('./routes/main'));
            }, 'main');
          },
          getChildRoutes: (nextState, cb) => {
            const lvl0 = [];
            foreachModule((module) => {
              if (!module.parent) {
                lvl0.push(module);
              }
            });
            createModuleRoutes(app, lvl0)
              .then((result) => {
                cb(null, result);
              })
              .catch((err) => {
                cb(err, null);
              });
          },
        },
      ],
    },
  ];
  return (
    <Router history={history} routes={routes} />
  );
}

RouterConfig.propTypes = {
  history: PropTypes.object,
  app: PropTypes.object,
};

export default RouterConfig;
